# -*- coding: utf-8 -*-

from django.contrib import admin
from django import forms

from .models import File


class FileForm(forms.ModelForm):

    def save(self, *a, **kw):
        kw.update({'commit': False})
        instance = super(FileForm, self).save(*a, **kw)

        # InMemoryUploadedFile at first save FileField otherwise
        data = self.cleaned_data['content']
        instance.mime_type = getattr(data, 'content_type',
            instance.mime_type)

        instance.save()
        return instance

    class Meta:
        model = File
        fields = '__all__'


@admin.register(File)
class FileAdmin(admin.ModelAdmin):

    list_display = ('slug', 'url', 'description')
    readonly_fields = ('url', 'preview', 'mime_type')
    search_fields = ('slug',)
    form = FileForm



# -*- coding: utf-8 -*-

from django.http import HttpResponse

from .models import File


def get_file(request, filename):

    f = File.objects.get(slug=filename)
    fs = open(f.content.path, 'r')
    response = HttpResponse(fs)
    response['Content-Disposition'] = "attachment; filename={} - {}".format(f.content, f.slug)
    return response




from django.conf.urls import url, include
from django.contrib import admin
from django.conf import settings
from django.conf.urls.static import static


urlpatterns = [
    url(r'^admin/', admin.site.urls),
    url(r'^djfiles/', include('djfiles.urls')),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# -*- coding: utf-8 -*-

import os

from slugify import slugify

from django.db import models
from django.db.models.signals import pre_delete, pre_save


def on_delete(sender, instance, **kwargs):
    for field in instance._meta.get_fields():
        if field and isinstance(field, models.FileField):
            fieldfield = getattr(instance, field.name)
            fieldfield.delete(False)


def on_save(sender, instance, **kwargs):
    if not instance.id:
        return

    if getattr(instance, '_pre_save_called_times', False):
        initial = sender.objects.get(pk=instance.id)

        for field in instance._meta.get_fields():
            if field and isinstance(field, models.FileField):
                new_value = getattr(instance, field.name)
                old_value = getattr(initial, field.name)
                if old_value and old_value != new_value:
                    old_value.delete(False)

    instance._pre_save_called_times = True


def cleanup(cls):
    uid = cls.__name__
    pre_delete.connect(receiver=on_delete, sender=cls, dispatch_uid=uid)
    pre_save.connect(receiver=on_save, sender=cls, dispatch_uid=uid)
    return cls


def file_upload_to(instance, filename):
    name, ext = os.path.splitext(filename)
    filename = '{}{}'.format(slugify(name, only_ascii=True), ext)
    return os.path.join('files', filename)


@cleanup
class File(models.Model):

    slug = models.SlugField(unique=True)
    mime_type = models.CharField(
        max_length=128, null=True, blank=True)
    content = models.FileField(upload_to=file_upload_to)
    description = models.TextField(null=True, blank=True)

    def __unicode__(self):
        return self.slug

    def preview(self):

        if self.mime_type:
            if self.mime_type.startswith('image/'):
                preview = u'<img src="{0}" style="max-width: 50%">'
                return preview.format(self.content.url)
            return self.mime_type
        return '-'

    preview.allow_tags = True

    def url(self):
        return u'<a href="{0}">{0}</a>'.format(self.content.url)
    url.allow_tags = True

    class Meta:
        db_table = 'files'
        verbose_name = 'file'
        verbose_name_plural = 'files'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-08 19:12
from __future__ import unicode_literals

from django.db import migrations, models
import djfiles.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(unique=True)),
                ('mime_type', models.CharField(blank=True, max_length=128, null=True)),
                ('content', models.FileField(upload_to=djfiles.models.file_upload_to)),
                ('description', models.TextField(blank=True, null=True)),
            ],
            options={
                'db_table': 'files',
                'verbose_name': '\u0444\u0430\u0439\u043b',
                'verbose_name_plural': '\u0444\u0430\u0439\u043b\u044b',
            },
        ),
    ]



# -*- coding: utf-8 -*-

import os
import io
from setuptools import setup, find_packages


with io.open('README.md', encoding='utf-8') as f:
    long_description = f.read()
try:
    import pypandoc
    long_description = pypandoc.convert(long_description, 'rst', 'md')
    long_description = long_description.replace('\r', '')
    with io.open('README.rst', mode='w', encoding='utf-8') as f:
        f.write(long_description)
except (ImportError, OSError):
    print("!!! Can't convert README.md - install pandoc and/or pypandoc.")


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-djfiles',
    version='0.2',
    packages=find_packages(exclude=['djfiles.tests']),
    include_package_data=True,
    license='BSD License',
    description='A simple Django app to upload files via admin panel.',
    long_description=long_description,
    url='https://github.com/luminousmen/djfiles',
    author='Bobrov Kirill',
    author_email='miaplanedo@gmail.com',
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'slugify',
    ],
)




import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = 'test'

DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'djfiles'
]

MIDDLEWARE_CLASSES = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'wsgi.application'


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True


STATIC_URL = '/static/'

# just for demo project
MEDIA_URL = '/files/'